from OBP_reliability_pillar.guard_duty.guard_duty_enabled import *


# returns the consolidated guard duty compliance
def guard_duty_compliance(self):
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside guard_duty :: guard_duty_compliance()")

    response = [
        guard_duty_enabled(self)
    ]

    return response


import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# returns the list of lambda functions
def list_lambda_functions(client) -> dict:
    """
    :param client:
    :param self:
    :return:
    """
    logger.info(" ---Inside lambdafn.utils :: list_lambda_functions")

    function_lst = []

    marker = ''
    while True:
        if marker == '' or marker is None:
            response = client.list_functions()
        else:
            response = client.list_functions(
                Marker=marker
            )
        for fn in response['Functions']:
            function_lst.append(fn)

        try:
            marker = response['NextMarker']
            if marker == '':
                break
        except KeyError:
            break

    return function_lst




from OBP_reliability_pillar.auto_scaling.launch_config_public_ip_disabled import *
from OBP_reliability_pillar.auto_scaling.asg_elb_healthcheck_required import *


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()

# checks autoscaling compliance
def auto_scaling_compliance(self) -> dict:
    logger.info(" ---Inside auto_scaling_compliance()")
    response = [
        launch_config_public_ip_disabled(self),
        asg_elb_healthcheck_required(self)
    ]

    return response


from botocore.exceptions import ClientError
import logging

from OBP_reliability_pillar.elastic_load_balancer.utils import list_elb

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


def cross_zone_load_balancing_enabled(self) -> dict:
    """

    :param self:
    :return:
    """
    logger.info(" ---Inside elastic_load_balancer :: cross_zone_load_balancing_enabled()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "Cross Zone Load Balancing Enabled"
    description = "Checks if cross zone load balancing is enabled or not"
    resource_type = "Elastic Load Balancer"

    regions = self.session.get_available_regions('elb')

    for region in regions:
        try:
            client = self.session.client('elb', region_name=region)
            elb_list = list_elb(self, region)

            for elb in elb_list:
                response = client.describe_load_balancer_attributes(
                    LoadBalancerName=elb['name']
                )
                if not response['LoadBalancerAttributes']['CrossZoneLoadBalancing']['Enabled']:
                    result = False
                    failReason = 'AWS ELB cross zone load balancing is not enabled'
                    offenders.append(elb['name'])

        except ClientError as e:
            logger.error("Something went wrong with the regions {}: {}".format(region, e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }


import logging

from botocore.exceptions import ClientError

from OBP_reliability_pillar.dynamodb.utils import list_dynamodb_tables

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# Checks compliance for dynamodb-pitr-enabled
def dynamodb_pitr_enabled(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside dynamodb :: dynamodb_pitr_enabled()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "Dynamodb pitr enabled"
    description = "Checks that point in time recovery (PITR) is enabled for Amazon DynamoDB tables"
    resource_type = "Dynamodb"

    regions = self.session.get_available_regions('dynamodb')

    for region in regions:
        try:
            client = self.session.client('dynamodb', region_name=region)

            dynamodb_tables = list_dynamodb_tables(client)
            for table in dynamodb_tables:
                response = client.describe_continuous_backups(
                    TableName=table
                )
                try:
                    status = response['ContinuousBackupsDescription']['PointInTimeRecoveryDescription']['PointInTimeRecoveryStatus']

                    if status == 'DISABLED':
                        result = False
                        failReason = "PITR is disabled"
                        offenders.append(table)

                except KeyError:
                    result = False
                    failReason = "PITR is disabled"
                    offenders.append(table)

        except ClientError as e:
            logger.error("Something went wrong with region {}: {}".format(region, e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }



import logging

from OBP_reliability_pillar.ec2.ec2_instance_detailed_monitoring_enabled import ec2_instance_detailed_monitoring_enabled
from OBP_reliability_pillar.ec2.instance_in_vpc import instance_in_vpc

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# returns consolidated dynamodb compliance
def ec2_compliance(self) -> list:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside ec2 :: ec2_compliance()")

    response = [
        ec2_instance_detailed_monitoring_enabled(self),
        instance_in_vpc(self)
    ]

    return response



def list_dynamodb_tables(client):
    table_names = []

    marker = ''
    while True:
        if marker == '' or marker is None:
            response = client.list_tables()
        else:
            response = client.list_tables(
                ExclusiveStartTableName=marker
            )
        table_names.extend(response['TableNames'])

        try:
            marker = response['LastEvaluatedTableName']
            if marker == '':
                break
        except KeyError:
            break

    return table_names




from OBP_reliability_pillar.rds.rds_automatic_minor_version_upgrade_enabled import *
from OBP_reliability_pillar.rds.rds_backup_enabled import rds_backup_enabled
from OBP_reliability_pillar.rds.rds_enhanced_monitoring_enabled import *
from OBP_reliability_pillar.rds.rds_multi_az_support_enabled import *
from OBP_reliability_pillar.rds.rds_instance_deletion_protection_enabled import *

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# returns consolidated dynamodb compliance
def rds_compliance(self) -> list:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside rds :: rds_compliance()")

    response = [
        rds_multi_az_support_enabled(self),
        rds_instance_deletion_protection_enabled(self),
        rds_enhanced_monitoring_enabled(self),
        rds_automatic_minor_version_upgrade_enabled(self),
        rds_backup_enabled(self),
    ]

    return response


import logging

from botocore.exceptions import ClientError

from OBP_reliability_pillar.lambdafn.utils import list_lambda_functions

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


def lambda_dlq_check(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside lambdafn :: lambda_dlq_check()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "Lambda DLQ check"
    description = "Checks whether an AWS Lambda function is configured with a dead-letter queue."
    resource_type = "AWS Lambda"

    regions = self.session.get_available_regions('lambda')

    for region in regions:
        try:
            client = self.session.client('lambda', region_name=region)
            function_lst = list_lambda_functions(client)

            for function in function_lst:
                response = client.get_function_configuration(
                    FunctionName=function['FunctionName']
                )
                try:
                    dead_letter_arn = response['DeadLetterConfig']
                    if dead_letter_arn is None:
                        raise KeyError
                except KeyError:
                    result = False
                    offenders.append(function['FunctionName'])
                    failReason = 'Amazon Lambda function is not associated with a Dead-Letter Queue (DLQ)'

        except ClientError as e:
            logger.error("Something went wrong with the region {}: {}".format(region, e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }


import logging

from botocore.exceptions import ClientError

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# Checks compliance for rds backup enabled
def rds_backup_enabled(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside rds :: rds_backup_enabled()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "RDS instance backup enabled"
    description = "Checks if RDS DB instances have backups enabled"
    resource_type = "RDS Instance"

    regions = self.session.get_available_regions('rds')

    for region in regions:
        try:
            client = self.session.client('rds', region_name=region)
            marker = ''
            while True:
                if marker == '' or marker is None:
                    response = client.describe_db_instances(
                        MaxRecords=100
                    )
                else:
                    response = client.describe_db_instances(
                        MaxRecords=100,
                        Marker=marker
                    )
                for instance in response['DBInstances']:
                    retention_period = instance['BackupRetentionPeriod']
                    if retention_period <= 0:
                        result = False
                        failReason = 'RDS Instance backup is not enabled'
                        offenders.append(instance['DBInstanceIdentifier'])

                try:
                    marker = response['Marker']
                    if marker == '':
                        break
                except KeyError:
                    break
        except ClientError as e:
            logger.error('Something went wrong with the region {}: {}'.format(region,e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }




from OBP_reliability_pillar.lambdafn.lambda_dlq_check import *
import logging

from OBP_reliability_pillar.lambdafn.lambda_inside_vpc import lambda_inside_vpc

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# checks aws lambda compliance
def lambda_compliance(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside lambdafn :: lambda_compliance()")
    response = [
        lambda_dlq_check(self),
        lambda_inside_vpc(self),
    ]

    return response


import logging

from botocore.exceptions import ClientError

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()

# checks compliance for redshift cluster maintenance settings check
def redshift_cluster_maintenance_settings_check(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside redshift :: redshift_cluster_maintenance_settings_check()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "Redshift cluster maintenance settings check"
    description = "Checks whether Amazon Redshift clusters have the specified maintenance settings"
    resource_type = "Redshift"

    regions = self.session.get_available_regions('redshift')

    for region in regions:
        try:
            client = self.session.client('redshift', region_name=region)
            marker = ''
            while True:
                if marker == '' or marker is None:
                    response = client.describe_clusters()
                else:
                    response = client.describe_clusters(
                        Marker = marker
                    )
                for cluster in response['Clusters']:
                    version_upgrade = cluster['AllowVersionUpgrade']
                    window = cluster['PreferredMaintenanceWindow']
                    automated_snapshot_retention_period = cluster['AutomatedSnapshotRetentionPeriod']

                    if not version_upgrade and automated_snapshot_retention_period > 0:
                        result = False
                        failReason = 'allowVersionUpgrade is not enabled'
                        offenders.append(cluster['ClusterIdentifier'])

                try:
                    marker = response['Marker']
                    if marker == '':
                        break
                except KeyError:
                    break

        except ClientError as e:
            logger.error("Something went wrong with the region {}: {}".format(region, e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }





import logging
from OBP_reliability_pillar.redshift.redshift_backup_enabled import redshift_backup_enabled
from OBP_reliability_pillar.redshift.redshift_cluster_maintenancesettings_check import \
    redshift_cluster_maintenance_settings_check

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# returns consolidated dynamodb compliance
def  redshift_compliance(self) -> list:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside redshift :: redshift_compliance()")

    response = [
        redshift_backup_enabled(self),
        redshift_cluster_maintenance_settings_check(self),
    ]

    return response


# returns the consolidated elastic_search compliance
from OBP_reliability_pillar.elastic_search.elastic_search_in_vpc_only import *


def elastic_search_compliance(self):
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside elastic_search :: elastic_search_compliance()")

    response = [
        elastic_search_in_vpc_only(self)
    ]

    return response


import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()

# checks the compliance.py for s3 bucket versioning enabled
def s3_bucket_versioning_enabled(self):
    """

    :param self:
    :return dict: details of s3 bucket versioning enabled compliance.py
    """
    logger.info(" ---Inside s3 :: s3_bucket_versioning_enabled()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "S3 bucket versioning enabled"
    description = "Checks if bucket versioning is enabled in s3 buckets."
    resource_type = "S3 Buckets"

    client = self.session.client('s3')
    response = client.list_buckets()

    for bucket in response['Buckets']:
        bucket_name = bucket['Name']

        try:
            resp = client.get_bucket_versioning(
                Bucket=bucket_name,
            )
            status = resp['Status']
        except KeyError:
            result = False
            failReason = "Either bucket versioning is not enabled or configuration not found"
            offenders.append(bucket_name)
            continue

        if not status == 'Enabled':
            result = False
            failReason = "Either bucket versioning is not enabled or configuration not found"
            offenders.append(bucket_name)

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }


import logging
import botocore
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()

# checks compliance.py for rds automatic minor version enabled
def rds_enhanced_monitoring_enabled(self) -> dict:
    """

    :param self:
    :return dict: rds enhanced monitoring enabled compliance.py details
    """
    logger.info(" ---Inside rds :: rds_enhanced_monitoring_enabled()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "RDS instance enhanced monitoring enabled"
    description = "Checks if enhanced monitoring is enabled for RDS instances."
    resource_type = "RDS Instance"

    regions = self.session.get_available_regions('rds')

    for region in regions:
        try:
            client = self.session.client('rds', region_name=region)
            marker = ''
            while True:
                response = client.describe_db_instances(
                    MaxRecords=100,
                    Marker=marker
                )
                for instance in response['DBInstances']:
                    monitoring_interval = int(instance['MonitoringInterval'])
                    if monitoring_interval <= 0:
                        result = False
                        failReason = "enhanced monitoring is not enabled"
                        offenders.append(region+': '+instance['DBInstanceIdentifier'])

                try:
                    marker = response['Marker']
                    if marker == '':
                        break
                except KeyError:
                    break
        except botocore.exceptions.ClientError as e:
            logger.error('Something went wrong with region {}: {}'.format(region, e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }


from OBP_reliability_pillar.elastic_beanstalk.enhanced_health_reporting_enabled import *

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# returns consolidated dynamodb compliance
def elastic_beanstalk_compliance(self) -> list:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside elastic_beanstalk :: elastic_beanstalk_compliance()")

    response = [
        enhanced_health_reporting_enabled(self)
    ]

    return response


import botocore
import logging

from botocore.exceptions import ClientError

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


def enhanced_health_reporting_enabled(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside elastic_beanstalk :: enhanced_health_reporting_enabled()")

    regions = self.session.get_available_regions('elasticbeanstalk')

    result = True
    failReason = ''
    offenders = []
    compliance_type = "Beanstalk Enhanced Health Reporting Enabled"
    description = "Checks if an AWS Elastic Beanstalk environment is configured for enhanced health reporting. The rule is COMPLIANT if the environment is configured for enhanced health reporting."
    resource_type = "Elastic Beanstalk"

    for region in regions:
        try:
            client = self.session.client('elasticbeanstalk', region_name=region)
            marker = ''
            while True:
                if marker == '' or marker is None:
                    response_describe_eb = client.describe_environments()
                else:
                    response_describe_eb = client.describe_environments(
                        NextToken=marker
                    )
                for env in response_describe_eb['Environments']:
                    if len(env['HealthStatus']) == 0:
                        result = False
                        failReason = 'AWS Elastic Beanstalk environment is not configured for enhanced health reporting.'
                        offenders.append(env['EnvironmentId'])

                try:
                    marker = response_describe_eb['NextToken']
                    if marker == '':
                        break
                except KeyError:
                    break
        except ClientError as e:
            logger.error("Something went wrong with region {}: {}".format(region, e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }



from botocore.exceptions import ClientError
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# compliance.py check for cloudwatch alarm check
def alarm_action_check(self) -> dict:
    """

    :param self:
    :return:
    """
    logger.info(" ---Inside cloudwatch :: alarm_action_check()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "Cloudwatch Alarm check"
    description = "Checks whether CloudWatch alarms have at least one alarm action, one INSUFFICIENT_DATA action, or one OK action enabled."
    resource_type = "Cloudwatch"

    regions  = self.session.get_available_regions('cloudwatch')

    for region in regions:
        try:
            client = self.session.client('cloudwatch', region_name=region)
            marker = ''
            while True:
                if marker == '' or marker is None:
                    response_describe_alarms = client.describe_alarms()
                else:
                    response_describe_alarms = client.describe_alarms(
                        NextToken=marker
                    )
                for alarm in response_describe_alarms['CompositeAlarms']:
                    alarm_action = len(alarm['AlarmActions'])
                    insufficient_data_action = len(alarm['InsufficientDataActions'])
                    ok_action = len(alarm['OKActions'])

                    if not alarm_action or not insufficient_data_action or not ok_action:
                        result = False
                        failReason = 'CloudWatch alarms does not have at least one alarm action, one INSUFFICIENT_DATA action, or one OK action enabled.'
                        offenders.append(alarm['AlarmName'])


                for alarm in response_describe_alarms['MetricAlarms']:
                    alarm_action = len(alarm['AlarmActions'])
                    insufficient_data_action = len(alarm['InsufficientDataActions'])
                    ok_action = len(alarm['OKActions'])

                    if alarm_action or insufficient_data_action or ok_action:
                        result = False
                        failReason = 'CloudWatch alarms does not have at least one alarm action, one INSUFFICIENT_DATA action, or one OK action enabled.'
                        offenders.append(alarm['AlarmName'])

                try:
                    marker = response_describe_alarms['NextToken']
                    if marker == '':
                        break
                except KeyError:
                    break
        except ClientError as e:
            logger.error("Something went wrong with region {}: {}".format(region, e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }


import logging

from botocore.exceptions import ClientError

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# checks compliance for elastic search in vpc only
def elastic_search_in_vpc_only(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside elastic_search :: elastic_search_in_vpc_only()")

    result = True
    failReason = ""
    offenders = []
    compliance_type = "Elastic search in vpc"
    description = "Checks if Elasticsearch domains are in Amazon Virtual Private Cloud (Amazon VPC)"
    resource_type = "Elastic Search"

    regions = self.session.get_available_regions('es')

    for region in regions:
        try:
            client = self.session.client('es', region_name=region)
            domain_names = client.list_domain_names()

            for domain in domain_names['DomainNames']:
                desc = client.describe_elasticsearch_domain(
                    DomainName=domain['DomainName']
                )
                try:
                    endpoint = desc['Endpoint']
                    if not endpoint is None:
                        raise KeyError
                except KeyError:
                    result = False
                    failReason = "Elastic search domain does not reside in a vpc"
                    offenders.append(domain['DomainName'])

        except ClientError as e:
            logger.error("Something wrong with the region {}: {}".format(region, e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }



import logging

from botocore.exceptions import ClientError

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


def redshift_backup_enabled(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside redshift :: redshift_backup_enabled()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "Redshift Backup Enabled"
    description = "Checks if backup is enabled on redshift cluster or not"
    resource_type = "Redshift"

    regions = self.session.get_available_regions('redshift')

    for region in regions:
        try:
            client = self.session.client('redshift', region_name=region)
            marker = ''
            while True:
                if marker == '' or marker is None:
                    response = client.describe_clusters()
                else:
                    response = client.describe_clusters(
                        Marker=marker
                    )
                for cluster in response['Clusters']:
                    retention_period = cluster['AutomatedSnapshotRetentionPeriod']
                    if retention_period <= 0:
                        result = False
                        failReason = "Redshift backup is not enabled"
                        offenders.append(region + ': ' + cluster['ClusterIdentifier'])

                try:
                    marker = response['Marker']
                    if marker == '':
                        break
                except KeyError:
                    break
        except ClientError as e:
            logger.error("Something went wrong with region {}: {}".format(region, e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }


import logging

from OBP_reliability_pillar.dynamodb.dynamodb_autoscaling_enabled import dynamodb_autoscaling_enabled
from OBP_reliability_pillar.dynamodb.dynamodb_pitr_enabled import dynamodb_pitr_enabled

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# returns consolidated dynamodb compliance
def dynamodb_compliance(self) -> list:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside dynamodb :: dynamodb_compliance()")

    response = [
        dynamodb_autoscaling_enabled(self),
        dynamodb_pitr_enabled(self),
    ]

    return response


import logging

import botocore

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


def launch_config_public_ip_disabled(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside autoscaling :: launch_config_public_ip_disabled()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "Launch configuration public ip disabled"
    description = "Checks if public ip is disabled in launch configuration or not"
    resource_type = "Auto Scaling"

    regions = self.session.get_available_regions('autoscaling')

    for region in regions:
        try:
            client_asg = self.session.client('autoscaling', region_name=region)

            n_token = ''
            while True:
                if n_token == '' or n_token is None:
                    autoscaling_response = client_asg.describe_launch_configurations()
                else:
                    autoscaling_response = client_asg.describe_launch_configurations(
                        NextToken=n_token
                    )

                for lc in autoscaling_response['LaunchConfigurations']:
                    try:
                        if lc['AssociatePublicIpAddress']:
                            result = False
                            failReason = 'launch configuration public ip is enabled'
                            offenders.append(lc['LaunchConfigurationARN'])
                    except KeyError:
                        pass

                try:
                    n_token = autoscaling_response['NextToken']
                    if n_token == '':
                        break
                except KeyError:
                    break
        except botocore.exceptions.ClientError:
            pass

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }



import logging

from botocore.exceptions import ClientError

from OBP_reliability_pillar.elastic_load_balancer.utils import list_elb

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# checks compliance for elb deletion protection enabled
def elb_deletion_protection_enabled(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside elastic_load_balancer :: elb_deletion_protection_enabled()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "ELB Deletion Protection Enabled"
    description = "Checks whether an Elastic Load Balancer has deletion protection enabled"
    resource_type = "Elastic Load Balancer"

    regions = self.session.get_available_regions('elbv2')

    for region in regions:
        try:
            client = self.session.client('elbv2', region_name=region)
            elb_list = list_elb(self, region)

            for elb in elb_list:
                response = client.describe_load_balancer_attributes(
                    LoadBalancerArn=elb['arn']
                )
                for attr in response['Attributes']:
                    if attr['Key'] == 'deletion_protection.enabled':
                        if not attr['Value']:
                            result = False
                            failReason = 'AWS ELB deletion protection is not enabled'
                            offenders.append(elb['name'])

        except ClientError as e:
            logger.error("Something went wrong with the regions {}: {}".format(region, e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }


import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# returns the list of load balancers
def list_elb(self, region: str)-> list:
    logger.info(" ---Inside list_elb()")
    res = []
    client = self.session.client('elb', region_name= region)

    marker = ''
    while True:
        if marker == '' or marker is None:
            response = client.describe_load_balancers()
        else:
            response = client.describe_load_balancers(
                Marker=marker
            )
        for lb in response['LoadBalancerDescriptions']:
            temp = {
                'name': lb['LoadBalancerName'],
                'arn': lb['LoadBalancerArn']
            }
            res.append(temp)

        try:
            marker = response['NextMarker']
            if marker == '':
                break
        except KeyError:
            break

    return res



from boto3 import session

from OBP_reliability_pillar.cloudwatch import cloudwatch
from OBP_reliability_pillar.dynamodb import dynamodb
from OBP_reliability_pillar.elastic_beanstalk import elastic_beanstalk
from OBP_reliability_pillar.elastic_load_balancer import elb
from OBP_reliability_pillar.rds import rds
from OBP_reliability_pillar.ec2 import ec2
from OBP_reliability_pillar.redshift import redshift
from OBP_reliability_pillar.s3 import s3
from OBP_reliability_pillar.security_hub import security_hub
from OBP_reliability_pillar.auto_scaling import auto_scaling
from OBP_reliability_pillar.lambdafn import lambdafn
from OBP_reliability_pillar.guard_duty import guard_duty
from OBP_reliability_pillar.elastic_search import elastic_search


__version__ = '0.0.9'
__author__ = 'Dheeraj Banodha'

class aws_client(elb, dynamodb, cloudwatch, rds, guard_duty, elastic_search,
                 ec2, s3, elastic_beanstalk, redshift, auto_scaling, security_hub, lambdafn):
    def __init__(self, aws_access_key_id: str, aws_secret_access_key: str):
        """

        @param str aws_access_key_id: AWS Access Key ID
        @param str aws_secret_access_key: AWS Secret Access Key
        """

        self.session = session.Session(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key
        )

        elb.__init__(elb, self.session)
        dynamodb.__init__(dynamodb, self.session)
        cloudwatch.__init__(cloudwatch, self.session)
        rds.__init__(rds, self.session)
        ec2.__init__(ec2, self.session)
        s3.__init__(s3, self.session)
        elastic_beanstalk.__init__(elastic_beanstalk, self.session)
        redshift.__init__(redshift, self.session)
        security_hub.__init__(security_hub, self.session)
        auto_scaling.__init__(auto_scaling, self.session)
        lambdafn.__init__(lambdafn, self.session)
        guard_duty.__init__(guard_duty, self.session)
        elastic_search.__init__(elastic_search, self.session)


    # consolidate compliance.py details
    def get_compliance(self) -> list:
        """
        :return list: consolidated list  of compliance.py checks
        """
        compliance = []
        compliance.extend(self.dynamodb_compliance())
        compliance.extend(self.elb_compliance())
        compliance.extend(self.cloudwatch_compliance())
        compliance.extend(self.rds_compliance())
        compliance.extend(self.ec2_compliance())
        compliance.extend(self.s3_compliance())
        compliance.extend(self.elastic_beanstalk_compliance())
        compliance.extend(self.redshift_compliance())
        compliance.extend(self.auto_scaling_compliance())
        compliance.extend(self.security_hub_enabled())
        compliance.extend(self.lambda_compliance())
        compliance.extend(self.guard_duty_compliance())
        compliance.extend(self.elastic_search_compliance())

        return compliance







import logging

from botocore.exceptions import ClientError

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# Checks for the compliance s3 bucket default lock enabled
def s3_bucket_default_lock_enabled(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside s3 :: s3_bucket_default_lock_enabled")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "S3 bucket default lock enabled"
    description = "Checks whether Amazon S3 bucket has lock enabled, by default"
    resource_type = "S3"

    client = self.session.client('s3')

    response = client.list_buckets()

    for bucket in response['Buckets']:
        try:
            response = client.get_object_lock_configuration(
                Bucket=bucket['Name']
            )
            object_lock_status = response['ObjectLockConfiguration']['ObjectLockEnabled']
            if object_lock_status != 'Enabled':
                result = False
                failReason = 'Object lock is not enabled'
                offenders.append(bucket['Name'])
        except KeyError:
            result = False
            failReason = 'Configuration not found'
            offenders.append(bucket['Name'])
        except ClientError as e:
            result = False
            failReason = 'Configuration not found'
            offenders.append(bucket['Name'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }



from botocore.exceptions import ClientError

import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()

# check the compliance for guard duty enabled
def guard_duty_enabled(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside guard_duty :: guard_duty_enabled()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "Guard Duty Enabled"
    description = "Checks if Amazon GuardDuty is enabled in your AWS account and region"
    resource_type = "Guard Duty"

    regions = self.session.get_available_regions('guardduty')

    for region in regions:
        try:
            client = self.session.client('guardduty')
            detectors = []
            marker = ''
            while True:
                if marker == '' or marker is None:
                    response = client.list_detectors()
                else:
                    response = client.list_detectors(
                        NextToken=marker
                    )
                detectors.extend(response['DetectorIds'])

                try:
                    marker = response['NextToken']
                    if marker == '':
                        break
                except KeyError:
                    break

            if len(detectors) <= 0:
                result = False
                failReason = "Guard duty is not enabled"

        except ClientError as e:
            logger.error("Something went wrong with the region {}: {}".format(region, e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }


import logging
from OBP_reliability_pillar.cloudwatch.alarm_action_check import alarm_action_check

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()

# checks compliance.py for cloudwatch
def cloudwatch_compliance(self) -> dict:
    logger.info(" ---Inside cloudwatch_compliance()")
    response = [
        alarm_action_check(self)
    ]

    return response


import botocore
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class security_hub:
    def __init__(self, session):
        """

        :param session:
        """
        self.session = session

    # checks compliance.py for security hub enabled
    def security_hub_enabled(self) -> list:
        logger.info(" ---Inside security_hub_enabled()")

        result = True
        failReason = ''
        offenders = []
        compliance_type = "security hub enabled"
        description = "Checks if security hub is enabled or not"
        resource_type = "Security hub"

        client = self.session.client('securityhub')
        try:
            response = client.describe_hub()
            # Scenario 1: SecurityHub is enabled for an AWS Account
            if response:
                pass
        except botocore.exceptions.ClientError as error:
            # Scenario 2: SecurityHub is not enabled for an AWS account.
            if error.response['Error']['Code'] == 'InvalidAccessException':
                result = False
                offenders = []
                failReason = "Security hub is disabled"


        return [{
            'Result': result,
            'failReason': failReason,
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description
        }]


import botocore
import logging

from OBP_reliability_pillar.dynamodb.utils import list_dynamodb_tables

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# checks compliance.py for dynamodb auto-scaling is enabled
def dynamodb_autoscaling_enabled(self) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside dynamodb :: dynamodb_autoscaling_enabled()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "Dynamodb autoscaling enabled"
    description = "Checks if Auto Scaling or On-Demand is enabled on your DynamoDB tables"
    resource_type = "Dynamodb"

    regions = self.session.get_available_regions('dynamodb')

    for region in regions:
        try:
            client = self.session.client('dynamodb',region_name=region)

            dynamodb_tables = list_dynamodb_tables(client)
            for table in dynamodb_tables:
                response = client.describe_table(
                    TableName=table
                )
                global_secondary_index = [index_name['IndexName'] for index_name in response['table']['GlobalSecondaryIndexes']]
                response_scalable_targets = client.describe_scalable_targets(
                    ServiceNamespace='dynamodb',
                    ResourceIds= [table].extend(global_secondary_index)
                )
                if len(response_scalable_targets['ScalableTargets']) == 0:
                    result = False
                    failReason = 'AWS DynamoDB Auto Scaling is not enabled for the table and/or its global secondary index.'
                    offenders.append(table)
        except botocore.exceptions.ClientError as e:
            logger.error("Something went wrong with region {}: {}".format(region, e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description
    }



import logging
from OBP_reliability_pillar.elastic_load_balancer.cross_zone_load_balancing_enabled import \
    cross_zone_load_balancing_enabled
from OBP_reliability_pillar.elastic_load_balancer.elb_deletion_protection import elb_deletion_protection_enabled

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# returns consolidated dynamodb compliance
def elb_compliance(self) -> list:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside elastic_load_balancer :: elb_compliance()")

    response = [
        cross_zone_load_balancing_enabled(self),
        elb_deletion_protection_enabled(self)
    ]

    return response